@@ -31,63 +31,8 @@
 #include "libks/ks.h"
 #include "libks/internal/ks_pool.h"
 
-#ifdef KS_DEBUG_POOL
-
-/**
- * Context structure for packing pool info
- */
-struct ks_debug_pool_pack_ctx_s {
-	void *user_data;
-	ks_json_t *pools_object;
-	ks_debug_pool_pack_type_t type;
-	ks_size_t alloc_c;
-	ks_size_t user_alloc;
-	ks_size_t max_alloc;
-	ks_size_t total_count;
-	ks_bool_t new_only;
-};
-
 typedef struct ks_debug_pool_pack_ctx_s ks_debug_pool_pack_ctx_t;
 
-static void global_debug_pool_add(struct ks_pool_s *pool)
-{
-	/* Wrap it in a debug handle to track them, pass 0 for size indicating we
-	 * are providing our own memory address (so we don't recurse in pool tracking) */
-	ks_handle_alloc(KS_HTYPE_DBG_POOL, 0, &pool, &pool->handle_base.handle, NULL);
-}
-
-static void global_debug_pool_remove(struct ks_pool_s *pool)
-{
-	ks_handle_destroy(&pool->handle_base.handle);
-}
-
-static ks_status_t global_debug_pool_iterate(ks_status_t (*callback)(struct ks_pool_s *pool, ks_debug_pool_pack_ctx_t *ctx), ks_debug_pool_pack_ctx_t *ctx)
-{
-	ks_handle_t handle = KS_NULL_HANDLE;
-	ks_status_t err = KS_STATUS_SUCCESS;
-
-	while (!ks_handle_enum_type(KS_HTYPE_DBG_POOL, &handle)) {
-		ks_pool_t *pool;
-		if (ks_handle_get(KS_HTYPE_DBG_POOL, pool->handle_base.handle, &pool))
-			continue;
-
-		if (NULL != ctx ) {
-			ctx->alloc_c += pool->alloc_c;
-			ctx->user_alloc += pool->user_alloc;
-			ctx->max_alloc += pool->max_alloc;
-			ctx->total_count++;
-		}
-
-		err = callback(pool, ctx);
-
-		ks_handle_put(KS_HTYPE_DBG_POOL, &pool);
-	}
-
-	return KS_STATUS_SUCCESS;
-}
-
-#endif
-
 static KS_THREAD_LOCAL uint32_t g_default_scanned_value = 0;
 
 static ks_status_t check_pool(const ks_pool_t *pool);
@@ -343,7 +288,6 @@ static void *alloc_mem(ks_pool_t *pool, const ks_size_t size, const char *file,
 	prefix->magic2 = KS_POOL_PREFIX_MAGIC;
 	prefix->refs = 1;
 	prefix->next = pool->first;
-	ks_assert(prefix->allocation_lock.count == 0);
 
 #ifdef KS_DEBUG_POOL
 	prefix->file = file;
@@ -410,12 +354,6 @@ static ks_status_t free_mem(void *addr)
 
 	CHECK_PREFIX(prefix);
 
-	/* Prevent a free on locked memory blocks */
-	if (!ks_spinlock_try_acquire(&prefix->allocation_lock)) {
-		ks_log(KS_LOG_ERROR, "Encountered locked block %p, preventing allocation\n", prefix);
-		return KS_STATUS_POOL_PREFIX_LOCKED;
-	}
-
 	pool = prefix->pool;
 
 	if (prefix->refs > 0) {
@@ -427,7 +365,6 @@ static ks_status_t free_mem(void *addr)
 	}
 
 	if (prefix->refs > 0) {
-		ks_spinlock_release(&prefix->allocation_lock);
 		return KS_STATUS_REFS_EXIST;
 	}
 
@@ -499,10 +436,6 @@ static ks_pool_t *ks_pool_raw_open(const ks_size_t flags, const char *file, int
 	pool->tag = tag;
 	pool->magic2 = KS_POOL_MAGIC;
 
-#ifdef KS_DEBUG_POOL
-	global_debug_pool_add(pool);
-#endif
-
 	SET_POINTER(error_p, KS_STATUS_SUCCESS);
 	return pool;
 }
@@ -601,19 +534,13 @@ static ks_json_t * __pack_pool_stats(ks_pool_t *pool, ks_debug_pool_pack_type_t
 		);
 
 		if (!(pool_heap_group_object= ks_json_get_object_item(heap_stats_object, workspace))) {
-			if (!(pool_heap_group_object = ks_json_add_item_to_object(heap_stats_object, workspace, ks_json_create_object()))) {
-				err = KS_STATUS_NO_MEM;
-				goto error;
-			}
+			pool_heap_group_object = ks_json_add_object_to_object(heap_stats_object, workspace);
 		}
 
 		/* Now make sure the heap stats array exists */
 		if (type == KS_DEBUG_POOL_PACK_TYPE_POOL_HEAP) {
 			if (!(heap_stat_array = ks_json_get_object_item(pool_heap_group_object, "allocation_pointers"))) {
-				if (!(heap_stat_array = ks_json_add_item_to_object(pool_heap_group_object, "allocation_pointers", ks_json_create_object()))) {
-					err = KS_STATUS_NO_MEM;
-					goto error;
-				}
+				heap_stat_array = ks_json_add_object_to_object(pool_heap_group_object, "allocation_pointers");
 			}
 		}
 
@@ -636,17 +563,6 @@ static ks_json_t * __pack_pool_stats(ks_pool_t *pool, ks_debug_pool_pack_type_t
 		snprintf(workspace2, sizeof(workspace2), "%p", (void *)addr);
 
 		ks_json_add_item_to_object(heap_stat_array, workspace2, ks_json_create_string(ks_thr_sprintf("[%s]", workspace3)));
-
-		/* Update the allocation count in this group */
-		if (!(count_number = ks_json_get_object_item(pool_heap_group_object, "allocation_count"))) {
-			if (!(count_number = ks_json_add_item_to_object(pool_heap_group_object, "allocation_count", ks_json_create_number(1)))) {
-				goto error;
-			}
-		} else {
-			double *count = ks_json_value_number_doubleptr(count_number);
-			*count = *count + 1;
-		}
-
 		index++;
 	}
 
@@ -681,18 +597,9 @@ static ks_status_t __pack_pool_callback(struct ks_pool_s *pool, ks_debug_pool_pa
 	}
 
 	// Fill it in
-	if (!ks_json_add_item_to_object(pool_object, "flags", ks_json_create_string_fmt("%ld", pool->flags))) {
-		err = KS_STATUS_NO_MEM;
-		goto error;
-	}
-	if (!ks_json_add_item_to_object(pool_object, "user_alloc", ks_json_create_string(ks_human_readable_size(pool->user_alloc, 1, sizeof(workspace), workspace)))) {
-		err = KS_STATUS_NO_MEM;
-		goto error;
-	}
-	if (!ks_json_add_item_to_object(pool_object, "max_alloc", ks_json_create_string(ks_human_readable_size(pool->max_alloc, 1, sizeof(workspace), workspace)))) {
-		err = KS_STATUS_NO_MEM;
-		goto error;
-	}
+	ks_json_add_item_to_object(pool_object, "flags", ks_json_create_string_fmt("%ld", pool->flags));
+	ks_json_add_item_to_object(pool_object, "user_alloc", ks_json_create_string(ks_human_readable_size(pool->user_alloc, 1, sizeof(workspace), workspace)));
+	ks_json_add_item_to_object(pool_object, "max_alloc", ks_json_create_string(ks_human_readable_size(pool->max_alloc, 1, sizeof(workspace), workspace)));
 
 	if (ctx->type == KS_DEBUG_POOL_PACK_TYPE_POOL || ctx->type == KS_DEBUG_POOL_PACK_TYPE_POOL_HEAP) {
 		heap_stats_object = __pack_pool_stats(pool, ctx->type, ctx->new_only);
@@ -738,24 +645,10 @@ static ks_status_t __pack_pool_summary(ks_json_t *object, ks_debug_pool_pack_ctx
 		return err;
 	}
 
-	if (!ks_json_add_item_to_object(summary_object, "alloc_c", ks_json_create_string_fmt("%ld", summary->alloc_c))) {
-		err = KS_STATUS_NO_MEM;
-		goto error;
-	}
-
-	if (!ks_json_add_item_to_object(summary_object, "user_alloc", ks_json_create_string(ks_human_readable_size(summary->user_alloc, 1, sizeof(workspace), workspace)))) {
-		err = KS_STATUS_NO_MEM;
-		goto error;
-	}
-	if (!ks_json_add_item_to_object(summary_object, "max_alloc", ks_json_create_string(ks_human_readable_size(summary->max_alloc, 1, sizeof(workspace), workspace)))) {
-		err = KS_STATUS_NO_MEM;
-		goto error;
-	}
-	if (!ks_json_add_item_to_object(summary_object, "total_count", ks_json_create_string_fmt("%lu", summary->total_count))) {
-		err = KS_STATUS_NO_MEM;
-		goto error;
-	}
-
+	ks_json_add_item_to_object(summary_object, "alloc_c", ks_json_create_string_fmt("%ld", summary->alloc_c));
+	ks_json_add_item_to_object(summary_object, "user_alloc", ks_json_create_string(ks_human_readable_size(summary->user_alloc, 1, sizeof(workspace), workspace)));
+	ks_json_add_item_to_object(summary_object, "max_alloc", ks_json_create_string(ks_human_readable_size(summary->max_alloc, 1, sizeof(workspace), workspace)));
+	ks_json_add_item_to_object(summary_object, "total_count", ks_json_create_string_fmt("%lu", summary->total_count));
 	ks_json_add_item_to_object(object, "summary", summary_object);
 
 error:
@@ -880,10 +773,6 @@ static ks_status_t ks_pool_raw_close(ks_pool_t *pool)
 		pool->log_func(pool, KS_POOL_FUNC_CLOSE, 0, 0, NULL, NULL, 0);
 	}
 
-#ifdef KS_DEBUG_POOL
-	global_debug_pool_remove(pool);
-#endif
-
 	ks_mutex_destroy(&pool->mutex);
 
 	free(pool);
@@ -1256,7 +1145,7 @@ done:
  *
  * DESCRIPTION:
  *
- * Ref count increment an address in a memoory pool.
+ * Ref count increment an address in a memory pool.
  *
  * RETURNS:
  *
@@ -1300,57 +1189,6 @@ done:
 	return addr;
 }
 
-KS_DECLARE(void) ks_pool_allocation_lock_release(void *addr)
-{
-	ks_status_t ret = KS_STATUS_SUCCESS;
-	ks_pool_prefix_t *prefix = NULL;
-	ks_pool_t *pool = NULL;
-
-	ks_assertd(addr);
-
-	prefix = (ks_pool_prefix_t *)((uintptr_t)addr - KS_POOL_PREFIX_SIZE);
-	CHECK_PREFIX(prefix);
-
-	pool = prefix->pool;
-	ks_assertd(!(ret = check_pool(pool)));
-
-	ks_spinlock_release(&prefix->allocation_lock);
-}
-
-KS_DECLARE(void) ks_pool_allocation_lock_acquire(void *addr)
-{
-	ks_status_t ret = KS_STATUS_SUCCESS;
-	ks_pool_prefix_t *prefix = NULL;
-	ks_pool_t *pool = NULL;
-
-	ks_assertd(addr);
-
-	prefix = (ks_pool_prefix_t *)((uintptr_t)addr - KS_POOL_PREFIX_SIZE);
-	CHECK_PREFIX(prefix);
-
-	pool = prefix->pool;
-	ks_assertd(!(ret = check_pool(pool)));
-
-	ks_spinlock_acquire(&prefix->allocation_lock);
-}
-
-KS_DECLARE(ks_bool_t) ks_pool_allocation_lock_try_acquire(void *addr)
-{
-	ks_status_t ret = KS_STATUS_SUCCESS;
-	ks_pool_prefix_t *prefix = NULL;
-	ks_pool_t *pool = NULL;
-
-	ks_assertd(addr);
-
-	prefix = (ks_pool_prefix_t *)((uintptr_t)addr - KS_POOL_PREFIX_SIZE);
-	CHECK_PREFIX(prefix);
-
-	pool = prefix->pool;
-	ks_assertd(!(ret = check_pool(pool)));
-
-	return ks_spinlock_try_acquire(&prefix->allocation_lock);
-}
-
 /*
  * void *__ks_pool_resize_ex
  *
@@ -1640,110 +1478,110 @@ KS_DECLARE(const char *) ks_pool_strerror(const ks_status_t error)
 
 KS_DECLARE(char *) __ks_pstrdup(ks_pool_t *pool, const char *str, const char *file, int line, const char *tag)
 {
-    char *result;
-    ks_size_t len;
+	char *result;
+	ks_size_t len;
 
-    if (!str) {
-        return NULL;
-    }
+	if (!str) {
+		return NULL;
+	}
 
-    len = (ks_size_t)strlen(str) + 1;
-    result = __ks_pool_alloc(pool, len, file, line, tag);
-    memcpy(result, str, len);
+	len = (ks_size_t)strlen(str) + 1;
+	result = __ks_pool_alloc(pool, len, file, line, tag);
+	memcpy(result, str, len);
 
-    return result;
+	return result;
 }
 
 KS_DECLARE(char *) __ks_pstrndup(ks_pool_t *pool, const char *str, ks_size_t len, const char *file, int line, const char *tag)
 {
-    char *result;
-    const char *end;
+	char *result;
+	const char *end;
 
-    if (!str) {
-        return NULL;
-    }
-
-    end = memchr(str, '\0', len);
-
-    if (!end) {
-        len = end - str;
+	if (!str) {
+		return NULL;
 	}
 
-    result = ks_pool_alloc(pool, len + 1);
-    memcpy(result, str, len);
-    result[len] = '\0';
+	end = memchr(str, '\0', len);
 
-    return result;
+	if (!end) {
+		len = end - str;
+	}
+
+	result = ks_pool_alloc(pool, len + 1);
+	memcpy(result, str, len);
+	result[len] = '\0';
+
+	return result;
 }
 
 KS_DECLARE(char *) __ks_pstrmemdup(ks_pool_t *pool, const char *str, ks_size_t len, const char *file, int line, const char *tag)
 {
-    char *result;
+	char *result;
 
-    if (!str) {
-        return NULL;
-    }
+	if (!str) {
+		return NULL;
+	}
 
-    result = __ks_pool_alloc(pool, len + 1, file, line, tag);
-    memcpy(result, str, len);
-    result[len] = '\0';
+	result = __ks_pool_alloc(pool, len + 1, file, line, tag);
+	memcpy(result, str, len);
+	result[len] = '\0';
 
-    return result;
+	return result;
 }
 
 KS_DECLARE(void *) __ks_pmemdup(ks_pool_t *pool, const void *buf, ks_size_t len, const char *file, int line, const char *tag)
 {
-    void *result;
+	void *result;
 
-    if (!buf) {
+	if (!buf) {
 		return NULL;
 	}
 
-    result = __ks_pool_alloc(pool, len, file, line, tag);
-    memcpy(result, buf, len);
+	result = __ks_pool_alloc(pool, len, file, line, tag);
+	memcpy(result, buf, len);
 
-    return result;
+	return result;
 }
 
 KS_DECLARE(char *) __ks_pstrcat(const char *file, int line, const char *tag, ks_pool_t *pool, ...)
 {
-    char *endp, *argp;
+	char *endp, *argp;
 	char *result;
-    ks_size_t lengths[10] = { 0 };
-    int i = 0;
-    ks_size_t len = 0;
-    va_list ap;
+	ks_size_t lengths[10] = { 0 };
+	int i = 0;
+	ks_size_t len = 0;
+	va_list ap;
 
-    va_start(ap, pool);
+	va_start(ap, pool);
 
 	/* get lengths so we know what to allocate, cache some so we don't have to double strlen those */
 
-    while ((argp = va_arg(ap, char *))) {
+	while ((argp = va_arg(ap, char *))) {
 		ks_size_t arglen = strlen(argp);
-        if (i < 10) lengths[i++] = arglen;
-        len += arglen;
-    }
+		if (i < 10) lengths[i++] = arglen;
+		len += arglen;
+	}
 
-    va_end(ap);
+	va_end(ap);
 
-    result = (char *) __ks_pool_alloc(pool, len + 1, file, line, tag);
-    endp = result;
+	result = (char *) __ks_pool_alloc(pool, len + 1, file, line, tag);
+	endp = result;
 
-    va_start(ap, pool);
+	va_start(ap, pool);
 
-    i = 0;
+	i = 0;
 
-    while ((argp = va_arg(ap, char *))) {
-        len = (i < 10) ? lengths[i++] : strlen(argp);
-        memcpy(endp, argp, len);
-        endp += len;
-    }
+	while ((argp = va_arg(ap, char *))) {
+		len = (i < 10) ? lengths[i++] : strlen(argp);
+		memcpy(endp, argp, len);
+		endp += len;
+	}
 
-    va_end(ap);
+	va_end(ap);
 
-    *endp = '\0';
+	*endp = '\0';
 
-    return result;
+	return result;
 }
 
 KS_DECLARE(char *) __ks_psprintf(const char *file, int line, const char *tag, ks_pool_t *pool, const char *fmt, ...)
