@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018 SignalWire, Inc
+ * Copyright (c) 2018-2023 SignalWire, Inc
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -88,6 +88,12 @@ KS_DECLARE(ks_status_t) ks_socket_option(ks_socket_t socket, int option_name, ks
 	case TCP_NODELAY:
 	case SO_KEEPALIVE:
 	case SO_LINGER:
+#ifdef KS_KEEP_IDLE_INTVL
+#ifndef __APPLE__
+	case TCP_KEEPIDLE:
+	case TCP_KEEPINTVL:
+#endif
+#endif /* KS_KEEP_IDLE_INTVL */
 #ifdef WIN32
 		result = setsockopt(socket, SOL_SOCKET, option_name, (char *) &opt, sizeof(opt));
 #else
